#!/usr/bin/env python

"""Usage: pontoon event show <id>

Options:
    -h --help       Show this page.
"""

from docopt import docopt
from digitalocean import Manager
from .. import ui, configure
from ..command import Command
from .. import MOCK


class EventCommand(Command):

    def __init__(self, config, args):
        self.config = config
        self.args = args
        self.manager = Manager(token=config['api_token'], mocked=MOCK)

    def show(self):
        action = self.manager.get_action(self.args['<id>'])
        details = ui.format_event(action)

        ui.message("Event %s" % self.args['<id>'])
        for k, v in details.items():
            ui.message("   %s: %s" % (k, v))
        return 0


def main():
    try:
        configure.logger()

        config = configure.combined()

        args = docopt(str(__doc__))

        exit(EventCommand(config, args).run())

    except Exception as e:
        ui.message(str(e))
        exit(1)

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

import os
import logging
import sys
from os.path import expanduser

try:
    from os import getuid
except ImportError:
    getuid = None

from subprocess import call, Popen, PIPE, CalledProcessError
from . import debug
from .exceptions import ConfigureException
from digitalocean import Manager, SSHKey
from . import MOCK

user_cfg = os.path.join(os.path.expanduser('~'), '.pontoon')

if sys.platform != "win32":
    sys_cfg = '/etc/pontoon.conf'
else:
    sys_cfg = ""

local_cfg = os.path.join(os.getcwd(), '.pontoon')
debug_mode = True if os.environ.get("DEBUG") else False
logformat = '%(asctime)s [%(name)s:%(levelname)s] %(message)s'
config_format = {
    'api_token': "",
    'auth_key': "",
    'auth_key_name': "",
}

defaults = {}
defaults['username'] = {
    'value': 'root',
    'title': 'Droplet login username',
}


def logger():
    """Prepare interface to logging."""
    logger = logging.getLogger('pontoon')
    formatter = logging.Formatter(logformat)

    handler = logging.StreamHandler()
    handler.setFormatter(formatter)
    logger.setLevel(logging.DEBUG if debug_mode else False)
    logger.addHandler(handler)
    return logger


@debug
def ssh_tools():
    """Checks for existance of SSH tools required for creating keys."""
    try:
        ssh_keygen = Popen(['command', '-v', 'ssh-keygen'],
                           stdout=PIPE, shell=True)
        openssl = Popen(['command', '-v', 'openssl'],
                        stdout=PIPE, shell=True)
        if ssh_keygen and openssl:
            return True
    except CalledProcessError:
        pass
    return False


@debug
def regenerate_public_key(private_key, public_key):
    """Generate an SSH public key from an existing private key."""
    private_key = expanduser(private_key)
    public_key = expanduser(public_key)
    openssl_keygen = ['openssl', 'rsa', '-in', private_key,
                      '-pubout', '>', public_key]
    call(openssl_keygen)


@debug
def rsa_keygen(private_key):
    """Create a new SSH key"""
    private_key = expanduser(private_key)
    rsa_keygen = ['ssh-keygen', '-t', 'rsa', '-N', '', '-f', private_key]
    call(rsa_keygen)


@debug
def combined():
    """Merge configuration defaults with values from config file."""
    config = {}
    for k, v in defaults.items():
        config[k] = v['value']
    for k, v in read_config().items():
        config[k] = v

    return config


@debug
def register_key(credentials, name, public_key):
    """Register an SSH key with Digital Ocean"""
    try:
        sshkey = SSHKey(token=credentials['api_token'], mocked=MOCK)
        if not sshkey.load_by_pub_key(public_key):
            sshkey.name = name
            sshkey.public_key = public_key
            sshkey.create()
    except Exception as e:
        raise ConfigureException(str(e))


def images(credentials):
    """Retrieve image options"""
    try:
        manager = Manager(token=credentials['api_token'], mocked=MOCK)
        return manager.get_global_images()
    except Exception as e:
        raise ConfigureException(str(e))


def sizes(credentials):
    """Retrieve size options"""
    try:
        manager = Manager(token=credentials['api_token'], mocked=MOCK)
        return manager.get_all_sizes()
    except Exception as e:
        raise ConfigureException(str(e))


def regions(credentials):
    """Retrieve region options"""
    try:
        manager = Manager(token=credentials['api_token'], mocked=MOCK)
        return manager.get_all_regions()
    except Exception as e:
        raise ConfigureException(str(e))


@debug
def read_key(path):
    """Read a public SSH key from the filesystem"""
    path = expanduser(path)
    try:
        with open(path, 'r') as f:
            return f.read()
    except IOError as e:
        raise ConfigureException(str(e))


@debug
def create_config(data):
    """Create a YAML config file from a dictionary"""
    import yaml
    data = yaml.dump(data, default_flow_style=False)

    if sys.platform != "win32":
        if getuid() == 0:  # root
            loc = sys_cfg
        else:
            loc = user_cfg
    else:
        loc = user_cfg

    with open(loc, 'w') as f:
        f.write(data.encode('UTF-8'))


@debug
def read_config():
    """Read a YAML formatted config into a dictionary"""
    import yaml
    config = config_format

    # sys_cfg empty in windows, but shouldnt affect this
    for loc in [local_cfg, user_cfg, sys_cfg]:
        try:
            with open(loc) as source:
                raw_config = source.read()
                if raw_config:
                    config = yaml.load(raw_config)
                    for expected in list(config_format.keys()):
                        if expected not in config:
                            config[expected] = None
                    return config
        except (TypeError, IOError, AttributeError):
            pass

    return config



#!/usr/bin/env python

"""Usage:
          pontoon sshkey list
          pontoon sshkey add <name> <public-key-path>
          pontoon sshkey show <name>
          pontoon sshkey replace <name> <public-key-path>
          pontoon sshkey destroy <name>

Options:
    -h --help       Show this page.
"""

from docopt import docopt
from digitalocean import Manager, SSHKey
from .. import configure, ui
from ..command import Command
from .. import MOCK


class SSHKeyCommand(Command):

    def __init__(self, config, args):
        self.config = config
        self.args = args
        self.manager = Manager(token=config['api_token'], mocked=MOCK)

    def _get_sshkey(self, name):
        resources = self.manager.get_all_sshkeys()
        resource = [res for res in resources if res.name == name]

        if len(resource) > 1:
            ui.warning("Warning: multiple SSH keys with identical "
                       "names found. Actions on those keys "
                       "will fail until this is resolved in the web UI.")
            raise Exception("Multiple SSH keys named '%s'" % name)

        if len(resource) == 0:
            raise Exception("No SSH key named '%s'" % name)

        return resource[0]

    def list(self):
        keys = self.manager.get_all_sshkeys()
        ui.message("Available SSH keys:")
        for s in keys:
            ui.message(" - %s" % s.name)
        return 0

    def add(self):
        ui.message("Creating %s using key at %s..." % (
            self.args['<name>'], self.args['<public-key-path>']))

        public_key = configure.read_key(self.args['<public-key-path>'])
        sshkey = SSHKey(token=self.config['api_token'], mocked=MOCK)
        if not sshkey.load_by_pub_key(public_key):
            sshkey.name = self.args['<name>']
            sshkey.public_key = public_key
            sshkey.create()

    def show(self):
        key = self._get_sshkey(self.args['<name>'])
        ui.message(key.public_key)

    def replace(self):
        ui.message("Replacing contents of %s using key at %s..." % (
            self.args['<name>'], self.args['<public-key-path>']))
        public_key = configure.read_key(self.args['<public-key-path>'])

        sshkey = self._get_sshkey(self.args['<name>'])
        sshkey.public_key = public_key
        sshkey.edit()

    def destroy(self):
        ui.message("Destroying %s..." % self.args['<name>'])
        sshkey = self._get_sshkey(self.args['<name>'])
        sshkey.destroy()


def main():
    try:
        configure.logger()

        config = configure.combined()

        args = docopt(str(__doc__))

        exit(SSHKeyCommand(config, args).run())

    except Exception as e:
        ui.message(str(e))
        exit(1)

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-


class PontoonException(Exception):
    """An ambiguous error occurred while handling the command"""


class ClientException(PontoonException):
    """An error occurred during the API request"""


class SnapshotException(PontoonException):
    """An error occurred while processing the snapshot"""


class DropletException(PontoonException):
    """An error occurred managing the Droplet"""


class ConfigureException(PontoonException):
    """An error occurred while processing the configuration file"""


class SSHKeyException(PontoonException):
    """An error occurred processing the SSH key command"""


class ImageException(PontoonException):
    """An error occurred processing the image commnad"""


class RegionException(PontoonException):
    """An error occurred processing the region command"""


class SizeException(PontoonException):
    """An error occurred processing the size command"""


class EventException(PontoonException):
    """An error occurred processing the event"""



# -*- coding: utf-8 -*-

import re
import sys
import contextlib
from random import randrange
from datetime import datetime, timedelta

from .exceptions import ClientException

# Python 2/3 compatibility for capture_stdout
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO


class Data(object):
    pass


def timestamp(hours=0):
    """Mocked Digital Ocean timestamp"""
    return (datetime.utcnow() + timedelta(
        hours=hours)).strftime("%Y-%m-%dT%H:%M:%SZ")


def get_builtins():
    """Python 2.x and 3.x have different names for accessing builtins"""
    try:
        __import__('builtins')
        return 'builtins'
    except ImportError:
        return '__builtin__'


def _raise(ex=None):
    """Wrapper for exceptions so they can be thrown from inside lambdas"""
    exception = []
    if ex:
        exception.append(ex)
    if len(exception):
        raise exception.pop()


def event_response():
    return {
        'event_id': randrange(9999),
        'status': 'OK',
    }


@contextlib.contextmanager
def capture_stdout():
    """Captures STDOUT and turns it into an object"""
    old = sys.stdout
    capturer = StringIO()
    sys.stdout = capturer
    data = Data()
    yield data
    sys.stdout = old
    data.result = capturer.getvalue()



#!/usr/bin/env python

"""Usage: pontoon configure [--help]

Options:
    -h --help       Show this page.
"""

import logging
from docopt import docopt
from digitalocean import Manager
from .. import configure, ui
from ..command import Command
from .. import MOCK


class ConfigureCommand(Command):

    def __init__(self, config, args):
        self.config = config
        self.args = args
        self.manager = Manager(token=config['api_token'], mocked=MOCK)

    def interactive(args):
        config = {}
        default = {}
        default['ssh_private_key'] = '~/.ssh/pontoon'
        default['ssh_public_key'] = '~/.ssh/pontoon.pub'

        ui.heading("Configure your Digital Ocean account with pontoon.")
        current = configure.combined()

        existing_token = " (current: %s)" % ui.mask(
            current['api_token']) if current['api_token'] else ""

        api_token = ui.ask("Personal Access Token%s" % existing_token)

        if api_token:
            config['api_token'] = api_token
        else:
            config['api_token'] = current['api_token']

        if configure.ssh_tools():
            ui.notify("Pontoon can either use an existing SSH "
                      "key or generate a new one using OpenSSH "
                      "(Linux/BSD/Mac only)")
        else:
            ui.warning("Pontoon cannot find openssl, "
                       "so won't be able to create keys.")

        if ui.ask_yesno("Use an existing keypair?"):
            for key in ['private', 'public']:
                key_name = 'ssh_%s_key' % key
                auth_key = ui.ask("Path to SSH %s key" % key)
                if auth_key:
                    config[key_name] = auth_key
                elif current[key_name]:
                    config[key_name] = current[key_name]
                else:
                    config[key_name] = default[key_name]

                while not ui.valid_path(config[key_name]):
                    ui.message("No key found at %s, retry?" % config[key_name])
                    config[key_name] = ui.ask("Path to SSH private key")

        else:
            config['ssh_private_key'] = default['ssh_private_key']
            config['ssh_public_key'] = default['ssh_public_key']

            if ui.valid_path(
                config['ssh_private_key']) and ui.valid_path(
                    config['ssh_public_key']):
                ui.message("Existing keypair found, using those.")

            elif ui.valid_path(
                config['ssh_private_key']) and not ui.valid_path(
                    config['ssh_public_key']):
                ui.message("Regenerating public key...")
                configure.regenerate_public_key(
                    config['ssh_private_key'], config['ssh_public_key'])

            else:
                configure.rsa_keygen(config['ssh_private_key'])

        config['auth_key_name'] = ui.filename_from_path(
            config['ssh_private_key'])
        public_key = configure.read_key(config['ssh_public_key'])

        ui.message("Registering public key to Digital Ocean...")
        try:
            configure.register_key(config,
                                   config['auth_key_name'],
                                   public_key)
            ui.message("ok!")
        except Exception as e:
            ui.message(str(e))
            pass

        for option in ['size', 'region', 'image']:
            ui.message("Choose a default %s:" % option)
            option_choices = {}
            option_index = 1
            options = getattr(configure, "%ss" % option)(config)
            for o in options:
                option_choices[option_index] = o.slug
                ui.message(" %-3s %s" % (str(option_index) + '.', o.slug))
                option_index += 1
            selection = ui.ask("%s (1-%d)" % (option.title(),
                                              len(option_choices)))
            selection = int(selection) if selection.isdigit() else 0
            while selection not in option_choices.keys():
                selection = ui.ask("Invalid selection. Choose 1-%d" % (
                    len(option_choices)))
                selection = int(selection) if selection.isdigit() else 0
            config[option] = str(option_choices[selection])

        other = {}
        for k, v in configure.defaults.items():
            val = current[k] if k in current else v['value']
            other[k] = ui.ask("%s? (currently '%s')" % (v['title'], val))

        for k, v in other.items():
            config[k] = v if v else configure.defaults[k]['value']

        configure.create_config(config)

        return 0


def main():
    try:
        configure.logger()

        config = configure.combined()

        args = docopt(str(__doc__))

        exit(ConfigureCommand(config, args).run("interactive"))

    except Exception as e:
        ui.message(str(e))
        exit(1)

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

from . import ui


class Command:
    """
    Base class for CLI commands.

    Parses docstrings and initalizes client.
    """

    def run(self, override=None):
        """Run the command specified by the docstring.

        :param override: Run a command other than the one specified.
        """

        if override is None:
            command = next(
                (k for k, v in self.args.items()
                    if k in dir(self) and v), None)
        else:
            command = override

        try:
            return getattr(self, command)()
        except (KeyboardInterrupt, EOFError):
            return 0
        except KeyError:
            ui.message("No command '%s'" % command)
        return 1



#!/usr/bin/env python

"""Usage:
          pontoon droplet list [--detail]
          pontoon droplet create <name> [--size=<size>] [--image=<image>]
                                        [--region=<region>] [--keys=<key>...]
                                        [--private-networking]
                                        [--disable-virtio] [--no-wait]
          pontoon droplet ssh <name> [--user=<user>] [--key=<path-to-key>]
          pontoon droplet rename <from> <to> [--no-wait]
          pontoon droplet resize <name> <size> [--no-wait]
          pontoon droplet snapshot <droplet-name> <snapshot-name> [--no-wait]
          pontoon droplet show <name> [--field=<field>]
          pontoon droplet status <name>
          pontoon droplet destroy <name>
          pontoon droplet start <name> [--no-wait]
          pontoon droplet shutdown <name> [--no-wait]
          pontoon droplet reboot <name> [--no-wait]
          pontoon droplet restore <name> <snapshot-name> [--no-wait]
          pontoon droplet rebuild <name> <image-name> [--no-wait]
          pontoon droplet powercycle <name> [--yes] [--no-wait]
          pontoon droplet poweroff <name> [--yes] [--no-wait]
          pontoon droplet passwordreset <name> [--yes]
          pontoon droplet backups <name> [ --enable | --disable ]

Options:
    -h --help              Show this page.
    --detail               Show full Droplet info.
    --field=<field>        Retrieve specified field from Droplet output.
                           Access with dot notation:
                              e.g., --field=networks.v4.0.ip_address
    --size=<size>          Droplet RAM allocation. [default: {size}]
    --image=<image>        Droplet image. [default: {image}]
    --region=<region>      Droplet region. [default: {region}]
    --keys=<key>...        List of registered keys to add
                           to Droplet(s) [default: {keys}].
    --private-networking   Assign private address to Droplet (where available)
    --disable-virtio       Disable VirtIO. (not recommended)
    --user=<user>          Override configured username for SSH login.
    --key=<path-to-key>    Override configured private key for SSH login.
    --yes                  Skip questions, assume any prompts will
                           be answered in the affirmative.

Quick guide:

    Some common operations:

        pontoon droplet list            A list of your droplets
        pontoon droplet create foo      Create a droplet called "foo"
                                        with default settings.
        pontoon droplet ssh foo         SSH into a droplet with the
                                        configured account and SSH key.
        pontoon droplet destroy foo     Terminate a droplet and secure
                                        erase the underlying drive.
"""

import re
from subprocess import call
from docopt import docopt
from digitalocean import Manager, Droplet, SSHKey
from .. import configure, ui
from ..command import Command
from .. import MOCK


class DropletCommand(Command):

    def __init__(self, config, args):
        self.config = config
        self.args = args
        self.manager = Manager(token=config['api_token'], mocked=MOCK)

    def _get_droplet(self, name):
        droplets = self.manager.get_all_droplets()
        droplet = [droplet for droplet in droplets if droplet.name == name]

        if len(droplet) > 1:
            ui.warning("Warning: multiple Droplets with identical "
                       "hostnames found. Actions on those Droplets "
                       "will fail until this is resolved in the web UI.")
            raise Exception("Multiple Droplets named '%s'" % name)

        if len(droplet) == 0:
            raise Exception("No Droplet named '%s'" % name)

        return droplet[0]

    def _get_image(self, name):
        resources = self.manager.get_images()
        resource = [res for res in resources if res.name == name]

        if len(resource) > 1:
            ui.warning("Warning: multiple images with identical "
                       "names found. Actions on those images "
                       "will fail until this is resolved in the web UI.")
            raise Exception("Multiple images named '%s'" % name)

        if len(resource) == 0:
            raise Exception("No image named '%s'" % name)

        return resource[0]

    def _wait(self, event, droplet, status="completed"):
        if not self.args['--no-wait']:
            state = 'in-progress'
            while state != 'completed':
                actions = droplet.get_actions()
                for action in actions:
                    if action.id == event['action']['id']:
                        state = action.status
                ui.ticker()
            ui.message(status)

    def list(self):
        droplet_list = self.manager.get_all_droplets()
        if len(set(d.name for d in droplet_list)) != len(droplet_list):
            ui.warning("Warning: multiple Droplets with identical "
                       "hostnames found. Actions on those Droplets "
                       "will fail until this is resolved in the web UI.")

        for machine in droplet_list:

            if self.args['--detail']:
                ui.message(machine.name)
                details = ui.format_droplet_info(machine)
                ui.yaml_message(details)
            else:
                ui.message("%-15s (%s, %s, %s, %s, %s)" % (
                    machine.name + ':',
                    machine.size_slug,
                    machine.image['slug'],
                    machine.region['slug'],
                    machine.ip_address,
                    machine.status,
                ))

    def create(self):
        ui.message("Creating Droplet %s "
                   "(%s using %s in %s)..." % (self.args['<name>'],
                                               self.args['--size'],
                                               self.args['--image'],
                                               self.args['--region'],
                                               ))

        try:
            if self._get_droplet(self.args['<name>']):
                ui.message("Cannot create two Droplets with same name.")
                return 1
        except Exception as e:
            pass

        try:
            droplet = Droplet(token=self.config['api_token'], mocked=MOCK)
            ssh_keys = [k.id for k in
                        self.manager.get_all_sshkeys() if k.name in
                        self.args['--keys']]

            droplet.create(
                name=self.args['<name>'],
                size=self.args['--size'],
                image=self.args['--image'],
                region=self.args['--region'],
                ssh_keys=ssh_keys,
                private_networking=self.args['--private-networking'],
                disable_virtio=self.args['--disable-virtio'])
        except Exception as e:
            ui.message(str(e))
            return 1

        if not self.args['--no-wait']:
            state = 'in-progress'
            while state != 'completed':
                actions = droplet.get_actions()
                for action in actions:
                    if action.type == 'create':
                        state = action.status
                ui.ticker()
            ui.message('active')

    def ssh(self):
        droplet = self._get_droplet(self.args['<name>'])
        if self.args['--user']:
            username = self.args['--user']
        else:
            username = self.config.get('username')
        hostname = droplet.ip_address
        if self.args['--key']:
            auth_key = self.args['--key']
        else:
            auth_key = self.config.get('ssh_private_key')
        auth_key = ui.full_path(auth_key)

        options = ['ssh',
                   '-i', '%s' % auth_key,
                   '-o', 'StrictHostKeyChecking=no',
                   '-o', 'UserKnownHostsFile=/dev/null',
                   '-o', 'LogLevel=error',
                   '%s@%s' % (username, hostname)]
        return call(options)

    def rename(self):
        ui.message("Renaming from %s to %s..." % (
            self.args['<from>'], self.args['<to>']))
        droplet = self._get_droplet(self.args['<from>'])
        droplet.rename(self.args['<to>'])

    def resize(self):
        ui.message("Resizing %s to %s" % (
            self.args['<name>'], self.args['<size>']))
        try:
            droplet = self._get_droplet(self.args['<name>'])

            # Check whether Droplet is powered off
            if droplet.status != 'off':
                if ui.ask_yesno("Droplet must be shut down during "
                                "this process, proceed?"):
                    ui.message("Shutting down Droplet...")
                    event = droplet.shutdown()
                    self._wait(event, droplet, status="off")
                else:
                    return

            # Perform the resize
            ui.message("Resizing...")
            event = droplet.resize(self.args['<size>'])
            self._wait(event, droplet, status="resized")

            # Boot the Droplet on completion
            if ui.ask_yesno("Boot Droplet?"):
                ui.message("Booting...")
                event = droplet.power_on()
                self._wait(event, droplet)

        except Exception as e:
            ui.message("Failed to resize: %s" % str(e))
            return 1

    def snapshot(self):
        ui.message("Snapshotting %s as %s..." % (
                   self.args['<droplet-name>'], self.args['<snapshot-name>']))
        try:
            droplet = self._get_droplet(self.args['<droplet-name>'])

            # Check whether Droplet is powered off
            if droplet.status != 'off':
                if ui.ask_yesno("Droplet must be shut down during this process"
                                ", proceed?"):
                    ui.message("Shutting down Droplet...")
                    event = droplet.shutdown()
                    self._wait(event, droplet, status="shutdown")
                else:
                    return

            ui.message("Beginning snapshot...")
            event = droplet.take_snapshot(self.args['<snapshot-name>'])
            self._wait(event, droplet)

            # Boot the Droplet on completion
            if ui.ask_yesno("Boot Droplet?"):
                ui.message("Booting...")
                event = droplet.power_on()
                self._wait(event, droplet)

        except Exception as e:
            ui.message("Failed to snapshot: %s" % str(e))
            return 1

    def show(self):
        droplet = self._get_droplet(self.args['<name>'])
        details = ui.format_droplet_info(droplet)

        # Uses a dot notation (foo.bar.baz) to access droplet details
        # to arbitrary depth.
        if self.args['--field']:
            fields = []
            for f in self.args['--field'].split('.'):
                if f.isdigit():
                    fields.append(int(f))
                else:
                    fields.append(f)

            try:
                result = reduce(lambda d, k: d[k], fields, details)
            except Exception as e:
                ui.message("Could not access field '%s'" % (
                           self.args['--field']))
                return 1
            if isinstance(result, dict) or isinstance(result, list):
                ui.yaml_message(result)
            else:
                ui.message(result)
        else:
            ui.message(droplet.name)
            ui.yaml_message(details)

    def status(self):
        droplet = self._get_droplet(self.args['<name>'])
        ui.message(droplet.status)

    def destroy(self):
        ui.message("Destroying %s and scrubbing data..." % self.args['<name>'])
        droplet = self._get_droplet(self.args['<name>'])
        droplet.destroy()

    def start(self):
        ui.message("Starting %s..." % self.args['<name>'])
        droplet = self._get_droplet(self.args['<name>'])
        event = droplet.power_on()
        self._wait(event, droplet, status="active")

    def shutdown(self):
        ui.message("Shutting down %s" % self.args['<name>'])
        droplet = self._get_droplet(self.args['<name>'])
        event = droplet.shutdown()
        self._wait(event, droplet, status="shutdown")

    def reboot(self):
        ui.message("Rebooting %s" % self.args['<name>'])
        droplet = self._get_droplet(self.args['<name>'])
        event = droplet.reboot()
        self._wait(event, droplet, status="rebooted")

    def restore(self):
        ui.message("Restoring %s from snapshot %s..." % (
                   self.args['<name>'], self.args['<snapshot-name>']))
        droplet = self._get_droplet(self.args['<name>'])
        image = self._get_image(self.args['<snapshot-name>'])
        event = droplet.restore(image.id)
        self._wait(event, droplet, status="restored")

    def rebuild(self):
        ui.message("Rebuilding %s using %s..." % (
                   self.args['<name>'], self.args['<image-name>']))
        droplet = self._get_droplet(self.args['<name>'])
        image = self._get_image(self.args['<image-name>'])
        event = droplet.rebuild(image.id)
        self._wait(event, droplet, status="rebuilt")

    def powercycle(self):
        droplet = self._get_droplet(self.args['<name>'])
        if self.args['--yes']:
            ui.message('Powercycling %s...' % self.args['<name>'])
            event = droplet.power_cycle()
            self._wait(event, droplet, status="powercycled")
        else:
            ui.notify("Powercycling a server could cause processes not to "
                      "shut down correctly, and potentially data loss or "
                      "corruption. "
                      "The 'reboot' command is the recommended way to restart "
                      "a machine.")
            if ui.ask_yesno("Do you wish to continue?"):
                ui.message('Powercycling %s...' % self.args['<name>'])
                event = droplet.power_cycle()
                self._wait(event, droplet, status="powercycled")

    def poweroff(self):
        droplet = self._get_droplet(self.args['<name>'])
        if self.args['--yes']:
            ui.message('Powering off %s...' % self.args['<name>'])
            event = droplet.power_off()
            self._wait(event, droplet, status="poweroff")
        else:
            ui.notify("Powering off a server could cause processes not to "
                      "shut down correctly, and potentially data loss or "
                      "corruption. The 'shutdown' command is the recommended "
                      "way to turn off a machine.")
            if ui.ask_yesno("Do you wish to continue?"):
                ui.message('Powering off %s...' % self.args['<name>'])
                event = droplet.power_off()
                self._wait(event, droplet, status="poweroff")

    def passwordreset(self):
        droplet = self._get_droplet(self.args['<name>'])
        if self.args['--yes']:
            ui.message('Resetting root password for %s...' % (
                self.args['<name>']))
            event = droplet.reset_root_password()
            self._wait(event, droplet, status="reset")
            ui.message('You should receive an email shortly.')
        else:
            ui.notify("Resetting the root password requires a reboot.")
            if ui.ask_yesno("Do you want to continue?"):
                ui.message('Resetting root password for %s...' % (
                    self.args['<name>']))
                event = droplet.reset_root_password()
                self._wait(event, droplet, status="reset")
                ui.message('You should receive an email shortly.')

    def backups(self):
        action = ''
        if self.args['--enable']:
            ui.message("This action is no longer supported and will be "
                       "removed in future versions. Backups must be enabled "
                       "at Droplet creation time.")
            return 1
        elif self.args['--disable']:
            action = 'disable'
        else:
            ui.message("action must be either 'enable' or 'disable'")
            return 1

        ui.message("%s backups for %s..." % (action, self.args['<name>']))
        droplet = self._get_droplet(self.args['<name>'])
        event = droplet.disable_backups()
        self._wait(event, droplet, status="disabled")


def main():
    try:
        configure.logger()

        config = configure.combined()

        args = docopt(str(__doc__.format(
            size=config.get('size', None),
            image=config.get('image', None),
            region=config.get('region', None),
            keys=config.get('auth_key_name', None))))

        exit(DropletCommand(config, args).run())

    except Exception as e:
        ui.message(str(e))
        exit(1)

if __name__ == '__main__':
    main()



#!/usr/bin/env python

"""Pontoon. A Python CLI for Digital Ocean.

Usage:
    pontoon <command> [<args>...]
    [--version]
    [--help]

The top level pontoon commands are:
    configure   Interactive configuration for pontoon
    droplet     Interact with Droplets
    event       Events on or around Droplets
    image       Public machine images available for Droplets
    region      Regions for Droplets
    size        Droplet sizes
    snapshot    Private snapshots created
    sshkey      SSH key management

See 'pontoon help <command>' for more information on a specific command.

"""

from subprocess import call
from docopt import docopt
from ..meta import __version__
from .. import ui


def main():
    args = docopt(__doc__,
                  version=__version__,
                  options_first=True)

    argv = [args['<command>']] + args['<args>']
    try:
        if args['<command>'] in ['help', None]:
            if not args['<args>']:
                return call(['pontoon', '--help'])
            return call(['pontoon-%s' % args['<args>'][0], '--help'])
        else:
            return call(['pontoon-%s' % args['<command>']] + argv)
    except (KeyboardInterrupt, EOFError):
        return 0
    except OSError:
        ui.message("No command '%s'" % args['<command>'])
    return 1


if __name__ == '__main__':
    exit(main())



#!/usr/bin/env python

"""Usage: pontoon size list

Options:
    -h --help       Show this page.
"""

from docopt import docopt
from digitalocean import Manager, Size
from .. import configure, ui
from ..command import Command
from .. import MOCK


class SizeCommand(Command):

    def __init__(self, config, args):
        self.config = config
        self.args = args
        self.manager = Manager(token=config['api_token'], mocked=MOCK)

    def list(self):
        available = self.manager.get_all_sizes()
        ui.message("Available sizes:")
        for s in available:
            ui.message(" - %s" % s.slug)
        return 0


def main():
    try:
        configure.logger()

        config = configure.combined()

        args = docopt(str(__doc__))

        exit(SizeCommand(config, args).run())

    except Exception as e:
        ui.message(str(e))
        exit(1)

if __name__ == '__main__':
    main()



#!/usr/bin/env python

"""Usage:
          pontoon image list [--with-ids]
          pontoon image oses
          pontoon image show <name>

Options:
    --with-ids      Include ids in output. Useful for other software that uses
                    Digital Ocean ids for input (like Packer).
    -h --help       Show this page.
"""

from docopt import docopt
from digitalocean import Manager, Image
from .. import configure, ui
from ..command import Command
from .. import MOCK


class ImageCommand(Command):

    def __init__(self, config, args):
        self.config = config
        self.args = args
        self.manager = Manager(token=config['api_token'], mocked=MOCK)

    def _get_image(self, name):
        resources = self.manager.get_all_images()
        resource = [res for res in resources if res.name == name]

        if len(resource) > 1:
            ui.warning("Warning: multiple images with identical "
                       "names found. Actions on those images "
                       "will fail until this is resolved in the web UI.")
            raise Exception("Multiple images named '%s'" % name)

        if len(resource) == 0:
            raise Exception("No image named '%s'" % name)

        return resource[0]

    def list(self):
        available = self.manager.get_all_images()
        ui.message("Available images:")
        if self.args['--with-ids']:
            ui.message("   %-10s %-10s %s" % ("id", "distro", "name"))
            ui.line(length=40)
        else:
            ui.message("   %-10s %s" % ("distro", "name"))
            ui.line(length=40)

        for s in available:
            if self.args['--with-ids']:
                ui.message(" - %-10s %-10s %s" % (
                           str(s.id) + ':', s.distribution, s.name))
            else:
                ui.message(" - %-10s %s" % (s.distribution, s.name))
        return 0

    def show(self):
        image = self._get_image(self.args['<name>'])
        details = ui.format_item(image)
        ui.yaml_message(details)

    def oses(self):
        available = self.manager.get_all_images()
        ui.message("Available Operating Systems:")
        oses = []
        for image in available:
            if image.distribution not in oses:
                oses.append(image.distribution)
        for os in oses:
            ui.message(" - %s" % os)
        return 0


def main():
    try:
        configure.logger()

        config = configure.combined()

        args = docopt(str(__doc__))

        exit(ImageCommand(config, args).run())

    except Exception as e:
        ui.message(str(e))
        exit(1)

if __name__ == '__main__':
    main()



#!/usr/bin/env python

from pontoon.meta import (__version__, __description__, __author__,
                          __author_email__, __url__)
from glob import glob
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

packages = [
    'pontoon',
    'pontoon.cmd'
]

requires = open("requirements/base.txt").read().split()
dependency_links = open("requirements/links.txt").read().split()

scripts = glob('scripts/pontoon*')

setup(
    name='pontoon',
    version=__version__,
    description=__description__,
    long_description=open('docs/README.rst').read(),
    author=__author__,
    author_email=__author_email__,
    url=__url__,
    packages=packages,
    package_data={'': ['LICENSE']},
    package_dir={'pontoon': 'pontoon'},
    # scripts=scripts,
    entry_points={
        "console_scripts": [
            "pontoon = pontoon.cmd.pontoon:main",
            "pontoon-configure = pontoon.cmd.pontoon_configure:main",
            "pontoon-droplet = pontoon.cmd.pontoon_droplet:main",
            "pontoon-event = pontoon.cmd.pontoon_event:main",
            "pontoon-image = pontoon.cmd.pontoon_image:main",
            "pontoon-region = pontoon.cmd.pontoon_region:main",
            "pontoon-size = pontoon.cmd.pontoon_size:main",
            "pontoon-snapshot = pontoon.cmd.pontoon_snapshot:main",
            "pontoon-sshkey = pontoon.cmd.pontoon_sshkey:main"
        ]
    },
    include_package_data=True,
    install_requires=requires,
    dependency_links=dependency_links,
    license=open('LICENSE').read(),
    zip_safe=False,
    classifiers=(
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Topic :: Internet :: WWW/HTTP',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: Implementation :: PyPy',
    ),
)



# -*- coding: utf-8 -*-

import logging
import functools


def debug(obj):
    logger = logging.getLogger(obj.__module__)

    @functools.wraps(obj)
    def log(*args, **kwargs):
        key = str(args) + str(kwargs)
        logger.debug("%s: %s" % (obj.__name__, key))
        return obj(*args, **kwargs)
    return log



# -*- coding: utf-8 -*-

from __future__ import print_function

# Windows / missing-readline compat
try:
    import readline
except ImportError:
    pass

import yaml
import textwrap
from os.path import (isfile, expanduser,
                     basename, splitext, join)
from time import sleep
from sys import stdout

# Python 2.6 compatibility
try:
    from collections import OrderedDict
except ImportError:
    OrderedDict = dict

# Python 2/3 compatibility
try:
    user_input = raw_input
except:
    user_input = input


# Borrowed from http://stackoverflow.com/questions/5121931
def ordered_dump(data, stream=None, Dumper=yaml.Dumper, **kwds):
    class OrderedDumper(Dumper):
        pass

    def _dict_representer(dumper, data):
        return dumper.represent_mapping(
            yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG,
            data.items())
    OrderedDumper.add_representer(OrderedDict, _dict_representer)
    return yaml.dump(data, stream, OrderedDumper, **kwds)


def ticker():
    """A loading/waiting indicator.

    Sends a '.' to the screen, resets, and sleeps.
    """
    stdout.write('.')
    stdout.flush()
    sleep(1)


def ask_yesno(question):
    """Present a string as a yes/no question on an interactive prompt"""
    question = "%s (y/n)" % question
    if user_input("%-15s: " % question).strip().lower() == 'y':
        return True
    return False


def ask(question):
    """Present a question with freeform input on an interactive prompt"""
    question += ':'
    response = user_input("%-15s " % question)
    return response


def valid_path(path):
    """Check whether a given string resolves to a path on the filesystem"""
    return isfile(expanduser(path))


def full_path(path):
    """Expand a ~/ prefixed path to a full path"""
    return join(expanduser(path))


def filename_from_path(path):
    """Get a filename from a given path"""
    return splitext(basename(expanduser(path)))[0]


def mask(text, masker='*'):
    """Hide part of a string,

    If the input is small, hide the entire string.
    Otherwise, show only the last few characters.
    """
    length = len(text)
    coverage = length / 100.0 * 20
    if (coverage < 0.9):
        result = (masker * length)
        return result
    masked_length = int(length - coverage)
    result = "%s%s" % ((masker * masked_length), text[masked_length:])
    return result


def format_droplet_info(machine):
    """Present Droplet information in a more human parseable format"""

    d = OrderedDict()
    d['id'] = machine.id
    d['size'] = machine.size_slug
    d['image'] = machine.image['slug']
    d['region'] = machine.region['slug']
    d['ip_address'] = machine.ip_address
    d['status'] = machine.status

    # Fields we want to remove / replace
    redacted = ['token', 'end_point', 'image', 'region',
                'size', 'mock_data', 'mock_status', 'mocked']
    details = machine.__dict__
    for k, v in details.items():
        if k.startswith('_') or k in redacted:
            del details[k]

    for k, v in details.items():
        d[k] = v
    return d


def format_event(action):
    """Present event information in a more human parseable format"""
    e = OrderedDict()
    e['id'] = action.id
    e['type'] = action.type
    e['resource_type'] = action.resource_type
    e['started_at'] = action.started_at
    e['completed_at'] = action.completed_at

    details = action.__dict__
    redacted = ['token', 'end_point', 'region',
                'mock_data', 'mock_status', 'mocked']
    for k, v in details.items():
        if k.startswith('_') or k in redacted:
            del details[k]

    for k, v in details.items():
        e[k] = v

    return e


def format_item(item):
    """Present any item in more human parseable format"""
    i = OrderedDict()
    i['id'] = item.id
    i['name'] = item.name

    details = item.__dict__
    redacted = ['token', 'end_point',
                'mock_data', 'mock_status', 'mocked']
    for k, v in details.items():
        if k.startswith('_') or k in redacted:
            del details[k]

    for k, v in details.items():
        i[k] = v

    return i


def message(text):
    """Wrapper for the `print` function"""
    print(text)
    return text


def yaml_message(data):
    """ Formats output as ordered YAML """
    message(ordered_dump(data,
                         Dumper=yaml.SafeDumper,
                         default_flow_style=False))


def heading(text, boxwidth=60):
    """Create a 'heading' styled textbox"""
    return box(text, decor="-", decor_y="|", boxwidth=boxwidth)


def notify(text, boxwidth=60):
    """Create a 'notification' styled textbox"""
    return box(text, decor="*", boxwidth=boxwidth)


def warning(text, boxwidth=60):
    """Create a 'warning' styled textbox"""
    return box(text, decor="!", boxwidth=boxwidth)


def line(length=60, decor='-'):
    """Write a line"""
    print(decor * length)


def box(text, decor='*', decor_x=None, decor_y=None,
        boxwidth=60, borderwidth=2):
    """Create a formatted textbox for highlighting important information"""
    decor_x = decor_x if decor_x else decor
    decor_y = decor_y if decor_y else decor
    decor_x_width = len(decor_x) if len(decor_x) > 0 else 1
    decor_y_width = len(decor_y) if len(decor_y) > 0 else 1
    decor_x_multiplier = int(boxwidth / decor_x_width)
    decor_y_multiplier = decor_y_width * 2

    textwidth = boxwidth - decor_y_multiplier - (borderwidth * 2)
    text = textwrap.wrap(text, textwidth)
    text = "\n".join([
        "{decor_y}{text}{decor_y}".format(
            decor_y=decor_y, text=m.center(
                boxwidth - decor_y_multiplier)) for m in text])
    border_x = (decor_x * decor_x_multiplier)
    border_space = "{decor_y}{spacer}{decor_y}".format(
        decor_y=decor_y, spacer=(" " * (boxwidth - decor_y_multiplier)))
    spacing = (' ' * boxwidth)
    text = "\n".join([
        spacing, border_x, border_space, text,
        border_space, border_x, spacing])
    print(text)
    return text



# -*- coding: utf-8 -*-

from __future__ import print_function

import sys
import os
import io
from subprocess import CalledProcessError
from pytest import raises
import pytest
from mock import MagicMock, patch


# Put lib dir into path so can be tested without installing
lib_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
test_dir = os.path.dirname(os.path.realpath(__file__))
sys.path.insert(0, lib_dir)

from docopt import docopt
from docopt import DocoptExit
from pontoon.command import Command
from pontoon import ui
from pontoon import configure
from pontoon.mocking import (_raise, capture_stdout, event_response,
                             get_builtins)


_manager = MagicMock(name='Manager')

_input = MagicMock(name='_input')


_expanduser = MagicMock(name='expanduser')


_sleep = MagicMock(name='sleep')


_call = MagicMock(name='call')


_Popen = MagicMock(name='Popen')


_open = MagicMock(name='open')


_getuid = MagicMock(name='getuid')
_getuid.return_value = 1

_fake_config = {
            'api_token': 'foobar',
            'auth_key': '~/.ssh/foo',
            'auth_key_name': 'foo'}

@patch('pontoon.ui.sleep', _sleep)
@patch('pontoon.ui.user_input', _input)
class TestUI:

    thisfile = os.path.realpath(__file__)

    def test_ticker(self):
        with capture_stdout() as capture:
            ui.ticker()
        assert capture.result == ""

    def test_ask_yesno(self):
        _input.return_value = "y"
        assert ui.ask_yesno("Foo?") is True
        _input.return_value = "n"
        assert ui.ask_yesno("Bar?") is False

    def test_ask(self):
        _input.return_value = "bar"
        assert ui.ask("Foo") == 'bar'

    def test_valid_path(self):
        assert ui.valid_path(self.thisfile)

    def test_full_path(self):
        assert ui.full_path(self.thisfile) == self.thisfile

    def test_filename_from_path(self):
        assert ui.filename_from_path(self.thisfile) == 'test_models'

    def test_mask(self):
        assert ui.mask('tiny') == '****'
        assert ui.mask('somewhatlonger') == '***********ger'
        assert (ui.mask('reallylongperhapsabighashorsomethinglikethat') ==
                ('***********************************glikethat'))

    def test_format_droplet_info(self):
        # FIXME: needs lib mocking
        pass

    def test_format_event(self):
        # FIXME: needs lib mocking
        pass

    def test_message(self):
        with capture_stdout() as capture:
            ui.message("foo bar baz")
        assert capture.result == "foo bar baz\n"

    def test_heading(self):
        heading = (
            "------------------------------------------------------------\n"
            "|                                                          |\n"
            "|                           foo                            |\n"
            "|                                                          |\n"
            "------------------------------------------------------------\n"
        )
        with capture_stdout() as capture:
            ui.heading("foo")
        assert heading.strip() == capture.result.strip()

    def test_notify(self):
        notify = (
            "************************************************************\n"
            "*                                                          *\n"
            "*                           foo                            *\n"
            "*                                                          *\n"
            "************************************************************"
        )
        with capture_stdout() as capture:
            ui.notify("foo")
        assert notify.strip() == capture.result.strip()

    def test_warning(self):
        warning = (
            "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n"
            "!                                                          !\n"
            "!                           foo                            !\n"
            "!                                                          !\n"
            "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n"
        )
        with capture_stdout() as capture:
            ui.warning("foo")
        assert warning.strip() == capture.result.strip()


@patch('pontoon.configure.getuid', _getuid)
@patch('pontoon.configure.expanduser', _expanduser)
@patch('pontoon.configure.call', _call)
@patch('pontoon.configure.Popen', _Popen)
@patch('%s.open' % get_builtins(), _open)
class TestConfigure:

    def test_ssh_tools(self):
        _Popen.side_effect = ["/usr/bin/ssh-keygen", "/usr/bin/openssl"]
        assert configure.ssh_tools() is True

        _Popen.side_effect = lambda *args, **kwargs: (
            _raise(CalledProcessError("foo", "bar")))
        assert configure.ssh_tools() is False

    def test_regenerate_public_key(self):
        _call.return_value = None
        assert configure.regenerate_public_key("foo", "bar") is None

    def test_rsa_keygen(self):
        _call.side_effect = lambda x: print("generating a public key" % x)
        with capture_stdout() as capture:
            configure.rsa_keygen("foo")
        assert capture.result == "generating a public key\n"

    def test_combined(self):
        fake_config = (
            "api_token: foobar\n"
            "auth_key: ~/.ssh/foo\n"
            "auth_key_name: foo"
        ).encode('UTF-8')

        _open.side_effect = lambda *args, **kwargs: io.BytesIO(fake_config)
        assert configure.combined() == {
            'api_token': 'foobar',
            'auth_key': '~/.ssh/foo',
            'auth_key_name': 'foo',
            'username': 'root',
        }

    def test_register_key(self):
        # FIXME: needs lib mocking
        pass

    def test_read_key(self):
        key_response = 'bazbazbazbazbazbazbaz'.encode('UTF-8')
        _open.side_effect = lambda *args, **kwargs: io.BytesIO(key_response)
        key = configure.read_key('/foo/bar/baz')
        assert key == 'bazbazbazbazbazbazbaz'.encode('UTF-8')

    def test_create_config(self):
        _open.side_effect = lambda *args, **kwargs: io.BytesIO()
        assert configure.create_config({'foo': 'bar', 'baz': 'boo'}) is None
        _open.side_effect = lambda *args, **kwargs: io.BytesIO()
        _getuid.return_value = 0
        assert configure.create_config({'foo': 'bar', 'baz': 'boo'}) is None

    def test_read_config(self):
        fake_config = (
            "api_token: foobar\n"
            "auth_key: ~/.ssh/foo\n"
            "auth_key_name: foo"
        ).encode('UTF-8')

        _open.side_effect = lambda *args, **kwargs: io.BytesIO(fake_config)
        assert configure.read_config() == {
            'api_token': 'foobar',
            'auth_key': '~/.ssh/foo',
            'auth_key_name': 'foo',
        }
        fake_config = (
            "api_token: foobar\n"
            "auth_key: ~/.ssh/foo\n"
        ).encode('UTF-8')

        _open.side_effect = lambda *args, **kwargs: io.BytesIO(fake_config)
        assert configure.read_config() == {
            'api_token': 'foobar',
            'auth_key': '~/.ssh/foo',
            'auth_key_name': None,
        }
        _open.side_effect = lambda x: (_raise(IOError))
        configure.read_config() == {
            'api_token': 'foobar',
            'auth_key': '~/.ssh/foo',
            'auth_key_name': None,
        }


class TestCommand:

    def test_command(self):
        doc = """Usage: command foo"""
        config = _fake_config

        with raises(DocoptExit):
            args = docopt(str(doc))

        class SmallCommand(Command):

            def __init__(self, config, args):
                self.config = config
                self.args = args
                self.manager = _manager(token=config['api_token'])

            def foo(self):
                print("foo-answer")

            def bar(self):
                print("bar-answer")

        with capture_stdout() as capture:
            output = SmallCommand(config, docopt(str(doc), argv=['foo'])).run()
        assert capture.result == "foo-answer\n"

        with capture_stdout() as capture:
            output = SmallCommand(config, []).run("bar")
        assert capture.result == "bar-answer\n"
